# prc02u.py (encoding: utf-8)
  # 身長区分×性別のクロス集計表をカイ２乗検定にかける
  # 統計量等のほか、観測度数、調整済み残差も表示
  # docx(ワード文書)を書き出す
  # jinja2, pypandoc を利用
import os
import pandas as pd
from rpy2.robjects import r
from jinja2 import Template
from textwrap import dedent
import pypandoc
import pyrcmd as prc
prc.set_charset('utf-8')

src_file = 'data.csv'
rscript = dedent(u'''\
    dtf <- read.csv("%s", header=T, fileEncoding="%s")
    dtf$性別 <- factor(dtf$性別, levels=c("男性", "女性"),
      labels=c("男性", "女性"))
    cc <- cut(dtf$身長, breaks=c(-Inf, 155, 165, 175, Inf),
        labels=c("155未満", "155～165", "165～175", "175以上"), right=FALSE)
    dtf <- cbind(dtf, 身長区分=cc)
    tbl <- table(dtf$身長区分, dtf$性別)
    chi_test <- chisq.test(tbl)
    chi_test
    ''') % (src_file, prc.get_charset(src_file))
md_tpl = Template(dedent(u'''\
    # 「身長区分」×「性別」のカイ２乗検定結果\n
    　男女間の身長の違いをみるため、下の観測度数の表をカイ２乗検定にかけた。\n
    {{observed}}\n
    　検定結果の主な数値は次のとおり。\n
    - 統計量(カイ２乗値): {{X_squared}}
    - 自由度(df): {{df}}
    - 有意確率(p値): {{p_value}}\n
    　有意確率の値が低いことから(p<0.05)、
    「身長区分」と「性別」の関係は独立なものではなく、
    有意な関連があると判断される。\n
    　そこで、調整済み残差をみると下のとおり。\n
    {{stdres}}\n
    　これをみると、165未満では女性が有意に多く、
    一方、165以上では男性が有意に多いことが分かる。\n
    '''))

prc.rexec(rscript, sink=False)  # Rコマンドの実行
chi_test = r['chi_test']  # rp2オブジェクト(ListVector)として取得
pod = prc.rp2pd(chi_test)  # ListVectorを変換(OrderedDictになる)
fod = prc.format_od(pod)
md_src = md_tpl.render(fod)  # markdown原稿の生成
md_file = 'temp.md'
prc.write_text(md_file, md_src, 'utf-8')  # markdown原稿をファイルに
pypandoc.convert_file(md_file, 'docx',
    outputfile='prc02u.docx', format='md')
os.unlink(md_file)
